import numpy as np
import pandas as pd
import pytest

from toarstats.metrics.interface import calculate_statistics
from toarstats.metrics.constants import (
    ALLOWED_SAMPLING_VALUES, STATISTICS_LIST
)


def test_calculate_statistics_simple_input():
    data = pd.DataFrame(
        range(10), index=pd.date_range(start="2000", periods=10, freq="H")
    )
    result = calculate_statistics("daily", "mean", data)
    expected_result = pd.DataFrame(
        {"mean": 4.5}, index=pd.DatetimeIndex(["2000"], freq="D")
    )
    pd.testing.assert_frame_equal(result, expected_result)


def test_calculate_statistics_different_min_data_capture():
    data = pd.DataFrame(
        [1, 2, np.nan, 4, 5, np.nan, 7, np.nan, np.nan, np.nan],
        index=pd.date_range(start="2000", periods=10, freq="H")
    )
    result = calculate_statistics("daily", "mean", data, min_data_capture=0.75)
    expected_result = pd.DataFrame(
        {"mean": np.nan}, index=pd.DatetimeIndex(["2000"], freq="D")
    )
    pd.testing.assert_frame_equal(result, expected_result)

    data = pd.DataFrame(
        range(24), index=pd.date_range(start="2000", periods=24, freq="H")
    )
    result = calculate_statistics("daily", "diurnal_cycle", data)
    expected_result = pd.DataFrame(
        {"diurnal_cycle": [float(num) for num in range(24)]},
        index=pd.date_range(start="2000", periods=24, freq="H")
    )
    pd.testing.assert_frame_equal(result, expected_result)


@pytest.mark.parametrize("sampling", ALLOWED_SAMPLING_VALUES)
@pytest.mark.parametrize("statistic", STATISTICS_LIST)
@pytest.mark.slow
def test_calculate_statistics_run_all_statistics_with_all_samplings(
        sampling, statistic, data, metadata
):
    if statistic == "drmdmax1h" and sampling == "monthly":
        with pytest.raises(
                ValueError,
                match="The drmdmax1h statistic cannot be evaluated with"
                      " monthly sampling"
        ):
            result = calculate_statistics(sampling, statistic, data, metadata)
    else:
        result = calculate_statistics(sampling, statistic, data, metadata)
        if (("aot40" in statistic or "w126" in statistic)
              and sampling == "seasonal"):
            expected_num_columns = 2
        elif statistic == "diurnal_cycle" and sampling == "monthly":
            expected_num_columns = 12
        elif sampling == "seasonal":
            expected_num_columns = 4
        elif sampling == "vegseason":
            expected_num_columns = 2
        else:
            expected_num_columns = 1
        if statistic == "drmdmax1h" and sampling not in {"daily", "monthly"}:
            expected_num_columns *= 2
        elif statistic == "nvgtall":
            expected_num_columns *= 6
        elif statistic == "percentiles1":
            expected_num_columns *= 3
        elif statistic == "percentiles2":
            if sampling not in {"summer", "xsummer", "annual"}:
                expected_num_columns *= 7
            else:
                expected_num_columns *= 9
        assert len(result.columns) == expected_num_columns
